require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/meta_model/meta_model_create'
require 'uu/os/meta_model/meta_model_attributes'
require 'uu/os/meta_model/meta_model_entry_attributes'
require 'uu/os/meta_model/meta_model_set_attributes'
require 'uu/os/meta_model/meta_model_get_entry_list'

module UU
  module OS
    # Meta Model service.
    module MetaModel
      extend self

      # Service path
      PATH = 'ues/core/container/UESMetaModel'
      # Creates a new meta model. The command creates a new meta model in a specified location.
      # At the very least meta artifact and name must be specified in {MetaModelCreate}
      # DTO. Competent role for the meta model will be selected as the most suitable according
      # to specified container and meta artifact (executive/authorized role with connected
      # interface), or can be also specified in {MetaModelCreate} DTO. 
      #
      # @param [String, UU::OS::UESURI] location_uri UESURI of the folder/org. unit/meta model where meta model will be created. 
      # @param [MetaModelCreate] meta_model DTO containing attributes of the new meta model. 
      # @deprecated Use {UU::OS::AppPackage.create} instead.
      # @return [UU::OS::UESURI] UESURI of the new meta model.
      def create(location_uri, meta_model = nil)
        svc = UU::OS::REST::RemoteClient.new(MetaModel)
        payload = UU::OS::MetaModel::MetaModelCreate.new(meta_model).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', location_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # This command returns attributes representing the meta model specified by the
      # metaModelUri parameter. The command does not change the state of the meta model
      # in the system. 
      #
      # @param [String, UU::OS::UESURI] meta_model_uri UESURI of a meta model whose attributes are returned 
      # @deprecated Use {UU::OS::AppPackage.get_attributes} instead.
      # @return [MetaModelAttributes] DTO containing the meta model attributes 
      def get_attributes(meta_model_uri)
        svc = UU::OS::REST::RemoteClient.new(MetaModel)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', meta_model_uri)
          return MetaModel::MetaModelAttributes.new(res)
        end
      end

      # Command for setting attributes of a meta model. The command can't modify references
      # to objects like location, competent role and so on. 
      #
      # @param [String, UU::OS::UESURI] meta_model_uri UESURI of the meta model whose attributes are to be changed 
      # @param [MetaModelSetAttributes] meta_model DTO containing new attributes the meta model 
      # @deprecated Use {UU::OS::AppPackage.set_attributes} instead.
      # @return [UU::OS::UESURI] UESURI of the meta model
      def set_attributes(meta_model_uri, meta_model = nil)
        svc = UU::OS::REST::RemoteClient.new(MetaModel)
        payload = UU::OS::MetaModel::MetaModelSetAttributes.new(meta_model).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', meta_model_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # This command deletes the specified meta model from the system. The meta model is
      # deleted even if it contains a locked sheet or attachment. The meta model cannot
      # be deleted when: 
      #   the meta model is not empty
      #   the meta model's workflow is not created 
      #   a property contains a reference to this meta model 
      #   a time sheet exists with a reference to this meta model 
      # @deprecated Use {UU::OS::AppPackage.delete} instead.
      # @param [String, UU::OS::UESURI] meta_model_uri UESURI of the meta model which is about to be deleted.
      def delete(meta_model_uri)
        svc = UU::OS::REST::RemoteClient.new(MetaModel)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('delete', meta_model_uri)
        end
      end

      # This command returns a list of entries (artifacts, meta artifacts, meta interfaces,
      # containers, meta model dictionaries or shortcuts) enlisted in the specified
      # meta model dictionary. Command does not return all entries, only a sublist. The
      # list of returned entries could be filtered by the name, code or entityTypeUri of
      # an entry. The list is sorted by the name (and codes in case are equal) of a entry by
      # default, or it can be sorted arbitrarily by any of entry attributes usable for the
      # filtering or these attributes combinations. 
      # @deprecated Use {UU::OS::AppPackage.get_entry_list} instead.
      # @param [String, UU::OS::UESURI] meta_model_uri UESURI of the meta model dictionary from where entries are listed. 
      # @param [MetaModelGetEntryList] criteria DTO containing criteria. Available parameters are: 
      # {MetaModelEntryAttributes#name name}, {MetaModelEntryAttributes#code code} and 
      # {MetaModelEntryAttributes#entityTypeUri entityTypeUri}. 
      #
      # See {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation}.
      #
      # @return [UU::OS::REST::ResultList<MetaModel::MetaModelGetEntryList, MetaModel::MetaModelEntryAttributes>] List of DTO representing meta model dictionary entries.
      def get_entry_list(meta_model_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(MetaModel)
        dto = MetaModel::MetaModelGetEntryList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        svc.add_parameter('recursive', dto.recursive)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getEntryList', meta_model_uri)
          return UU::OS::REST::ResultList.new(MetaModel::MetaModelGetEntryList, MetaModel::MetaModelEntryAttributes, res)
        end
      end

    end
  end

end
