require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/organizational_unit/organizational_unit_create'
require 'uu/os/organizational_unit/organizational_unit_attributes'
require 'uu/os/organizational_unit/organizational_unit_entry_attributes'
require 'uu/os/organizational_unit/organizational_unit_set_attributes'
require 'uu/os/organizational_unit/organizational_unit_get_entry_list'

module UU
  module OS
    module OrganizationalUnit
      extend self

      # Service path
      PATH = 'ues/core/container/UESOrganizationalUnit'
      # Creates a new organizational unit. The command creates a new organizational unit
      # in a specified location. At the very least meta artifact must be specified in
      # {UU::OS::OrganizationalUnit::OrganizationalUnitCreate} DTO. Competent role for the organizational unit
      # will be selected as the most suitable according to specified container and meta
      # artifact (executive/authorized role with connected interface), or can be also
      # specified in {UU::OS::OrganizationalUnit::OrganizationalUnitCreate} DTO.
      #
      # @param [String, UU::OS::UESURI] location_uri URI of the folder/org. unit/meta model where organizational unit will be created.
      # @param [UU::OS::OrganizationalUnit::OrganizationalUnitCreate] organizational_unit Specific options.
      # @option organizational_unit [String, UU::OS::UESURI] :meta_artifact_uri URI of the meta artifact. Only a meta artifact with the template class of Organizational Unit (not its descendants) can be used. If the rights does not permit to create a new organizational unit from the meta artifact in the specified location, the create command fails. 
      # @option organizational_unit [String] :code Code of the new organizational unit. When nil is set, the code is generated automatically using the system sequence. The code must be unique in the given territory, otherwise the create command fails. 
      # @option organizational_unit [String] :name Name of the new organizational unit. When nil is set, tName from meta artifact is used if set, otherwise the create command fails.  
      # @option organizational_unit [String] :description Description of the new organizational unit. When nil is set, tDescription from meta artifact is used if set, otherwise no description is set on the new organizational unit. 
      # @option organizational_unit [String, UU::OS::UESURI] :competent_role_uri URI of the competent role. When nil is set, the executive role from authorization is used. If no role is authorized to create the new organizational unit into specified container, the create command fails. If the selected role doesn't have connected the role interface with the selected meta artifact, the create command fails. If the selected role doesn't belong to one of superior organizational units, the create command fails. 
      # @option organizational_unit [String, UU::OS::Artifact::SecurityLevel] :security_level Security level. When nil is set, security level is taken from the meta artifact tSecurityLevel. If the meta artifact tSecurityLevel is higher than the user's security clearance, the create command fails. 
      # @option organizational_unit [String, UU::OS::UESURI] :icon_uri URI of the organizational unit's icon. When nil is set, the organizational unit uses icon from meta artifact. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'. 
      # @option organizational_unit [Boolean] :dry_run Specifies whether this is only a test run to verify that the object can be created. The default value is false.
      # @option organizational_unit [Boolean] :component_content Defines whether the primarily displayed content of the created Organizational Unit is component content. Default value is false.
      #
      # @return [UU::OS::UESURI] URI of the new organizational unit 
      #
      # @example
      #   # Create a new organizational unit.
      #   UU::OS::OrganizationalUnit.create('ues:TERRITORY:ORGANIZATIONAL_UNIT', :code => 'NEW_ORGANIZATIONAL_UNIT', :name => 'New Organizational Unit')
      #  
      #   # Verifying that new organizational unit can be created.
      #   UU::OS::OrganizationalUnit.create('ues:TERRITORY:ORGANIZATIONAL_UNIT', :code => 'NEW_ORGANIZATIONAL_UNIT', :name => 'New Organizational Unit', :dryRun => true)
      def create(location_uri, organizational_unit = nil)
        svc = UU::OS::REST::RemoteClient.new(OrganizationalUnit)
        payload = UU::OS::OrganizationalUnit::OrganizationalUnitCreate.new(organizational_unit).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', location_uri, payload)
          # res is nil if dry_run was set to true. In that case, return nil (do nothing)
          if !(res.nil? || res.empty?)
            return UU::OS::UESURI.new(res)
          end
        end
      end

      # This command returns attributes representing the organizational unit specified
      # by the organizationalUnitUri parameter. The command does not change the state
      # of the organizational unit in the system.
      #
      # @param [String, UU::OS::UESURI] organizational_unit_uri URI of an organizational unit whose attributes are returned 
      # 
      # @return [OrganizationalUnitAttributes] DTO containing the organizational unit attributes
      #
      # @example
      #   # Get attributes of an organizational unit
      #   UU::OS::OrganizationalUnit.get_attributes('ues:TERRITORY:ORGANIZATIONAL_UNIT')
      def get_attributes(organizational_unit_uri)
        svc = UU::OS::REST::RemoteClient.new(OrganizationalUnit)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', organizational_unit_uri)
          return OrganizationalUnit::OrganizationalUnitAttributes.new(res)
        end
      end

      # Command for setting attributes of an organizational unit. The command can't modify
      # references to objects like location, competent role and so on.
      #
      # @param [String, UU::OS::UESURI] organizational_unit_uri URI of the organizational unit whose attributes are to be changed
      # @param [OrganizationalUnitSetAttributes] organizational_unit DTO containing new attributes the organizational unit
      # @option organizational_unit [String] :name The new name of the organizational unit. If nil is set, the name is not modified.
      # @option organizational_unit [String] :code The new code of the organizational unit. If nil is set, the code is not modified. The code must be unique in a given territory, otherwise the set_attributes command fails.
      # @option organizational_unit [String] :description The new description of the organizational unit. If nil is set, the description is not modified.
      # @option organizational_unit [String, UU::OS::Artifact::SecurityLevel] :security_level The new security level of the organizational unit. If nil is set, the security level is not modified. If the meta artifact tSecurityLevel is higher than the user's security clearance, the set_attributes command fails.
      # @option organizational_unit [String, UU::OS::UESURI] :icon_uri URI of the organizational unit's icon. If nil is set, the icon is not modified. If nil URI ("ues:{[-1]}:{[-1]}:") is set, the organizational unit icon is removed and the organizational unit uses icon from meta artifact. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option organizational_unit [Boolean] :component_content Defines whether the primarily displayed content of the created Organizational Unit is component content. Default value is false.
      #
      # @return [UU::OS::UESURI] URI of the organizational unit
      #
      # @example
      #   # Set a different name and security_level of an existing organizational unit.
      #   UU::OS::OrganizationalUnit.set_attributes('ues:TERRITORY:ORGANIZATIONAL_UNIT', :name => 'New Name of Organizational Unit', :security_level => UU::OS::Artifact::SecurityLevel::INTERNAL_USE)
      def set_attributes(organizational_unit_uri, organizational_unit = nil)
        svc = UU::OS::REST::RemoteClient.new(OrganizationalUnit)
        payload = UU::OS::OrganizationalUnit::OrganizationalUnitSetAttributes.new(organizational_unit).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', organizational_unit_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # This command deletes the specified organizational unit from the system. The organizational
      # unit is deleted even if it contains a locked sheet or attachment. The organizational
      # unit cannot be deleted when:
      # the org. unit is not empty,
      # the org. unit's workflow is not created,
      # a property contains a reference to this org. unit,
      # a time sheet exists with a reference to this org. unit.
      #
      # @param [String, UU::OS::UESURI] org_unit_uri URI of the organizational unit which is about to be deleted.
      # 
      # @example
      #   # Deletes an organizational unit.
      #   UU::OS::OrganizationalUnit.delete('ues:TERRITORY:ORGANIZATIONAL_UNIT')
      def delete(org_unit_uri)
        svc = UU::OS::REST::RemoteClient.new(OrganizationalUnit)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('delete', org_unit_uri)
        end
      end

      # This command returns a list of entries (artifacts or shortcuts) enlisted in the
      # specified organizational unit. Only entries listed exactly in the specified
      # organizational units are considered (so subordinate organizational units are
      # not considered). The command does not return all entries, only a sublist. The list
      # of returned entries could be filtered by the name, code or type of an entry. The list
      # is sorted by the name (and codes in case are equal) of a entry by default, or it can
      # be sorted arbitrarily by any of entry attributes usable for the filtering or these
      # attributes combinations. See the UES query documentation for more information.
      #
      # See {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation}.
      #
      # @param [String, UU::OS::UESURI] organizational_unit_uri URI of the organizational unit from where entries (artifacts or shortcuts) are listed. 
      # @param [OrganizationalUnitGetEntryList] criteria Specific options.
      # @option criteria [String] :query Query for filtering and ordering of the result list. It has to conform the ues query syntax (see the documentation below for more info). If it is not specified, no filtering is applied and entries are ordered by name of referenced artifact (and code and type as the second and the third criteria). Otherwise, it is possible to filter and sort the result list by the following attributes: {OrganizationalUnitEntryAttributes#name name}, {OrganizationalUnitEntryAttributes#code code} and {OrganizationalUnitEntryAttributes#type type} : {OrganizationalUnitEntryType}.
      # @option criteria [Boolean] :recursive Flag for recursive listing. It means that if the listed organizational unit contains inner folders and the recursive flag is true, entries of the all inner folders are in result too. If the recursive flag is nil or false, only entries exactly from the listed organizational unit are returned.
      #
      # @return [UU::OS::REST::ResultList<OrganizationalUnit::OrganizationalUnitGetEntryList, OrganizationalUnit::OrganizationalUnitEntryAttributes>] List of DTO representing entries of the organizational_unit_uri (artifacts or shortcuts)
      #
      # @example
      #   # Returns list of containers (folders, organizational units and meta models) from an organizational unit.
      #   UU::OS::OrganizationalUnit.get_entry_list('ues:TERRITORY:ORGANIZATIONAL_UNIT',
      #      :query => 'type=container')
      def get_entry_list(organizational_unit_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(OrganizationalUnit)
        dto = OrganizationalUnit::OrganizationalUnitGetEntryList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        svc.add_parameter('recursive', dto.recursive)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getEntryList', organizational_unit_uri)
          return UU::OS::REST::ResultList.new(OrganizationalUnit::OrganizationalUnitGetEntryList, OrganizationalUnit::OrganizationalUnitEntryAttributes, res)
        end
      end

    end
  end

end
