require 'uu/os/qos/qos_handler'
require 'uu/os/rest'
require 'uu/os/uesuri'
require 'uu/os/personal_access_role/personal_access_role_attributes'
require 'uu/os/personal_access_role/personal_access_role_set_attributes'
require 'uu/os/personal_access_role/personal_access_role_create'
require 'uu/os/personal_access_role/personal_access_role_get_role_interface_list'
require 'uu/os/personal_access_role/personal_access_role_remove_role_interface'
require 'uu/os/personal_access_role/personal_access_role_add_role_interface'
require 'uu/os/personal_access_role/personal_access_role_get_role_interface_list_item'
require 'uu/os/personal_access_role/personal_access_role_get_list'

module UU
  module OS
    module PersonalAccessRole
      extend self

      # Service path of the REST API.
      PATH = 'uu/os/PersonalAccessRole'

      # This command returns attributes representing the personal access role specified
      # by the +personal_access_role_uri+ parameter. The command does not change the state of the role
      # in the system.
      #
      # @param [String, UU::OS::UESURI] personal_access_role_uri UESURI of a personal access role whose attributes are returned
      #
      # @return [PersonalAccessRoleAttributes] DTO containing the personal access role attributes
      #
      # @example
      #   # Get attributes of specified personal access role.
      #   UU::OS::PersonalAccessRole.get_attributes('ues:TERRITORY:PERSONAL_ACCESS_ROLE')
      #
      def get_attributes(personal_access_role_uri)
        svc = UU::OS::REST::RemoteClient.new(PersonalAccessRole)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAttributes', personal_access_role_uri)
          return PersonalAccessRole::PersonalAccessRoleAttributes.new(res)
        end
      end

      # Command for setting attributes of a personal access role. The command can't modify references
      # to objects like location, competent role and so on.
      #
      # @param [String, UU::OS::UESURI] personal_access_role_uri UESURI of the personal access role whose attributes are to be changed
      # @param [PersonalAccessRoleSetAttributes] personal_access_role DTO containing new attributes the personal access role
      # @option personal_access_role [String] :first_name The new first name of the personal access role. When nil is set, the first name is not modified.
      # @option personal_access_role [String] :surname The new surname of the personal access role. When nil is set, the surname is not modified.
      # @option personal_access_role [String] :code The new code of the personal access role. When nil is set, the code is not modified. When blank string is set, the code is generated automatically using the system sequence. The code must be unique in a given territory, otherwise the setAttributes command fails.
      # @option personal_access_role [String] :description The new description of the personal access role. When nil is set, the description is not modified.
      # @option personal_access_role [UU::OS::Artifact::SecurityLevel] :security_level Security level of the new personal access role. When nil is set, security level is taken from the meta artifact SecurityLevel. If the meta artifact SecurityLevel is higher than the user's security clearance, the create command fails.
      # @option personal_access_role [UU::OS::Artifact::SecurityLevel] :security_clearance Security clearance of the new personal access role. When nil is set, the security clearance is not modified.
      # @option personal_access_role [Boolean] :searchable_in_territory Specifies if it is possible to search the personal access role in its territory. When nil is set, this attribute on the personal access role is not modified.
      # @option personal_access_role [String, UU::OS::UESURI] :icon_uri URI of the personal access role's icon. When nil is set, the icon is not modified. If nil UESURI ("ues:{[-1]}:{[-1]}:")  is set, the personal access role icon is removed and the personal access role uses icon from meta artifact. Allowed format of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}CODE_OF_ICON}'. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option personal_access_role [Boolean] :grant_all_from_organizational_unit <b>Deprecated.</b> Specifies if the personal access role will be included among All roles from organizational unit. When nil is set, this attribute on the personal access role is not modified.
      # @option personal_access_role [Boolean] :component_content Defines whether the primarily displayed content of the created Personal access role is component content. Default value is false.            
      # @option personal_access_role [String] :avatar_uri UESURI or URL of image which represents access role's avatar (user photo).  If set to an empty string, the value will be erased. When nil is set, the value is not modified.
      #
      # @return [UU::OS::UESURI] UESURI of the personal access role
      #
      # @example
      #   #Set attributes of personal access role.
      #   accessRoleUri = UU::OS::PersonalAccessRole.set_attributes('ues:TERRITORY:PERSONAL_ACCESS_ROLE', :first_name=>'John', :surname=>'Doe', :code=>'NEWACCESSCODE', :description=>'Some basic access role',:security_level=>UU::OS::Artifact::SecurityLevel::CONFIDENTIAL,:searchable_in_territory=>false)
      #
      def set_attributes(personal_access_role_uri, personal_access_role = nil)
        svc = UU::OS::REST::RemoteClient.new(PersonalAccessRole)
        payload = UU::OS::PersonalAccessRole::PersonalAccessRoleSetAttributes.new(personal_access_role).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('setAttributes', personal_access_role_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Creates a new personal access role. The command creates a new personal access role in a specified location.
      # At the very least meta artifact must be specified in {PersonalAccessRoleCreate}
      # DTO. Competent role for the new role will be selected as the most suitable according
      # to the specified container and meta artifact (executive/authorized role with
      # connected interface), or can be also specified in {PersonalAccessRoleCreate}
      # DTO.
      #
      # @param [String, UU::OS::UESURI] location_uri UESURI of the folder/org. unit/meta model where the personal access role will be created.
      # @param [PersonalAccessRoleCreate] personal_access_role DTO containing attributes of the new personal access role
      # @option personal_access_role [String, UU::OS::UESURI] :meta_artifact_uri UESURI of the meta artifact. Only a meta artifact with the template class of an Access role (not its descendants) can be used. If the rights does not permit to create a new personal access role from the meta artifact in the specified location, the create command fails.
      # @option personal_access_role [String] :universe_id Universe identifier of the user. When nil is set, the create command fails.
      # @option personal_access_role [String] :first_name First name of the new personal access role. When nil is set, the create command fails.
      # @option personal_access_role [String] :surname Surname of the new personal access role. When nil is set, the create command fails.
      # @option personal_access_role [String] :code  Code of the new personal access role. When nil is set, the code is generated automatically using the system sequence. The code must be unique in the given territory, otherwise the create command fails.
      # @option personal_access_role [String] :description Description of the new personal access role. When nil is set, description from meta artifact is used if set, otherwise no description is set on the new personal access role.
      # @option personal_access_role [String, UU::OS::UESURI] :competent_role_uri  UESURI of the competent role. When nil is set, the executive role from authorization is used. If no role is authorized to create the personal access role into specified container, the create command fails. If the selected role doesn't have connected the access role interface with the selected meta artifact, the create command fails. If the selected role doesn't belong to the same organizational unit as anew role or if it doesn't belong to one of superior organizational units, the create command fails.
      # @option personal_access_role [UU::OS::Artifact::SecurityLevel] :security_level Security level. When nil is set, security level is taken from the meta artifact default security level. If the meta artifact template security level is higher than the user's security clearance, the create command fails.
      # @option personal_access_role [UU::OS::Artifact::SecurityLevel] :security_clearance Security clearance of the new personal access role. When nil is set, the value {UU::OS::Artifact::SecurityLevel::NO_CONFIDENTIALITY NO_CONFIDENTIALITY} is used.
      # @option personal_access_role [Boolean] :searchable_in_territory Specifies if it is possible to search the new personal access role in its territory. When nil is set, SearchableInTerritory from meta artifact is used if set, otherwise the create command fails.
      # @option personal_access_role [String, UU::OS::UESURI] :icon_uri URI of the personal access role's icon. When nil is set, the icon is not modified. If nil UESURI ("ues:{[-1]}:{[-1]}:")  is set, the personal access role icon is removed and the role uses icon from meta artifact. Allowed format of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}CODE_OF_ICON}'. An example of icon URI: 'ues:#{{}system}:#{{}ues_v5.core_v1.codetable_v1.base_v1.CodeTable_Icons}:#{{}ART_077}'.
      # @option personal_access_role [Boolean] :grant_all_from_organizational_unit <b>Deprecated.</b> Specifies if the personal access role will be included among All roles from organizational unit. When true or nil is set, the personal access role will be included, otherwise it will not be included among All roles from organizational unit.
      # @option personal_access_role [Boolean] :force If set to true, new Personal Access Role will be created even if Personal Access Role in {UU::OS::Activity::ActivityStateType::FINAL FINAL} state for the same user exists in the given territory.
      # @option personal_access_role [Boolean] :component_content Defines whether the primarily displayed content of the created Personal access role is component content. Default value is false.            
      # @option personal_access_role [String] :avatar_uri UESURI or URL of image which represents access role's avatar (user photo). When nil is set, the access role will be created without avatar.
      #
      # @return [UU::OS::UESURI] UESURI of the new personal access role 
      # @raise [UESPersonalAccessRoleRTException]
      #
      # @example
      #   # Create personal access role.
      #   UU::OS::PersonalAccessRole.create('ues:UNI:FOLDER', :meta_artifact_uri=>'ues:TERRITORY:METAARTIFACT',:universe_id=>'37820001', :first_name=>'John', :surname=>'Doe', :code=>'DOECODE', :description=>'Some sample description',:searchable_in_territory=>true)
      #
      def create(location_uri, personal_access_role = nil)
        svc = UU::OS::REST::RemoteClient.new(PersonalAccessRole)
        payload = UU::OS::PersonalAccessRole::PersonalAccessRoleCreate.new(personal_access_role).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.post('create', location_uri, payload)
          return UU::OS::UESURI.new(res)
        end
      end

      # Returns list of the role interfaces that are connected to the specified access
      # role. The list of returned interfaces could be filtered by the name or code. Returned
      # list does not contain any objects filtered out by executed UESQuery. The list is
      # sorted by the name (and by code in case are equal) of an interface by default.
      #
      # @param [String, UU::OS::UESURI] personal_access_role_uri UESURI of the personal access role for which the connected role interfaces are to be listed
      # @param [PersonalAccessRoleGetRoleInterfaceList] criteria DTO containing criteria for listing personal access role interfaces. Available parameters are: {PersonalAccessRoleGetRoleInterfaceListItem#name name} and {PersonalAccessRoleGetRoleInterfaceListItem#code code}. See {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation}.
      # @option criteria [String] :query The UES query string. It has to conform the uesquery syntax (see documentation for more info). Filtering and order of the result list can be specified there. When nil is set, no filtering is applied and entries are ordered by name of casted subjects (and code as the second criteria). See {https://uu.unicornuniverse.eu/ues/sesm?&SessFree=ues:UNI-BT:UES.DOC%2FCMD_V1%2FUESQUERY UESQuery Documentation}.
      # @return [UU::OS::REST::ResultList<PersonalAccessRole::PersonalAccessRoleGetRoleInterfaceList, PersonalAccessRole::PersonalAccessRoleGetRoleInterfaceListItem>] List of DTO representing role interfaces which are connected to the specified personal access role
      #
      # @example
      #   # Get role interface list descend ordered by code.
      #   roleIfcList = UU::OS::PersonalAccessRole.get_role_interface_list('ues:TERRITORY:ACCESSROLE', :query=>'ORDER BY code DESC')
      #
      def get_role_interface_list(personal_access_role_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(PersonalAccessRole)
        dto = PersonalAccessRole::PersonalAccessRoleGetRoleInterfaceList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getRoleInterfaceList', personal_access_role_uri)
          return UU::OS::REST::ResultList.new(PersonalAccessRole::PersonalAccessRoleGetRoleInterfaceList, PersonalAccessRole::PersonalAccessRoleGetRoleInterfaceListItem, res)
        end
      end

      # Returns list of personal access roles in the given location with specified criteria.
      #
      # @param [String, UU::OS::UESURI] territory_uri UESURI of the territory, from which the access roles will be retrieved. Never nil.
      # @param [UU::OS::PersonalAccessRole::PersonalAccessRoleGetList] criteria Object containing paging and filtering criteria.
      # @option criteria [String] :query The query for filtering the result list. If it is not set, no filtering is applied and personal access roles are ordered by their {PersonalAccessRoleAttributes#first_name firstName}, {PersonalAccessRoleAttributes#surname surname} and {PersonalAccessRoleAttributes#code code}. Available parameter is: {PersonalAccessRoleAttributes#universe_id universeId}
      #
      # @return [UU::OS::REST::ResultList<PersonalAccessRole::PersonalAccessRoleGetList, PersonalAccessRole::PersonalAccessRoleAttributes>] List of personal access roles in the territory. Contains instances of PersonalAccessRoleAttributes.
      #
      # @example
      #   # UESURI of the UNI territory
      #   terUri = 'ues:UNI-BT:UNI-BT'
      #
      #   # Get personal access role with uid 7-8864-1 in UNI territory if exists
      #   list = UU::OS::PersonalAccessRole.get_personal_access_role_list(terUri, :query => "universeId = '7-8864-1'")
      #
      #   # Get list of all personal access roles in UNI territory
      #   list = UU::OS::PersonalAccessRole.get_personal_access_role_list(terUri)
      #
      def get_personal_access_role_list(territory_uri, criteria = nil)
        svc = UU::OS::REST::RemoteClient.new(PersonalAccessRole)
        dto = PersonalAccessRole::PersonalAccessRoleGetList.new(criteria)
        svc.add_parameter('pageIndex', dto.page_index)
        svc.add_parameter('pageSize', dto.page_size)
        svc.add_parameter('query', dto.query)
        UU::OS::QoS::QoSHandler.auto_retry do
          res = svc.get('getAccessRoleList', territory_uri)
          return UU::OS::REST::ResultList.new(PersonalAccessRole::PersonalAccessRoleGetList, PersonalAccessRole::PersonalAccessRoleAttributes, res)
        end
      end

      # The command removes role interface connection from the specified personal access role.
      # Only connected interfaces can be disconnected from the personal access role.
      #
      # @param [String, UU::OS::UESURI] personal_access_role_uri UESURI of the personal access role where the role interface will be disconnected
      # @param [PersonalAccessRoleRemoveRoleInterface] personal_access_role DTO containing role interface that to be disconnected
      # @option personal_access_role [String, UU::OS::UESURI] :role_interface_uri UESURI of the role interface that to be unconnected from the specified personal access role. When nil is set, the command fails.
      #
      # @example
      #   # Remove role interface for specified personal access role
      #   UU::OS::PersonalAccessRole.remove_role_interface('ues:TERRITORY:PERSONAL_ACCESS_ROLE',:role_interface_uri=>'ues:TERRITORY:ROLE_IFC')
      #
      def remove_role_interface(personal_access_role_uri, personal_access_role = nil)
        svc = UU::OS::REST::RemoteClient.new(PersonalAccessRole)
        payload = UU::OS::PersonalAccessRole::PersonalAccessRoleRemoveRoleInterface.new(personal_access_role).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('removeRoleInterface', personal_access_role_uri, payload)
        end
      end

      # This command creates new role interface connection to the specified personal access role.
      # The role interface has to be in an active state and mustn't be already connected.
      # The role interface has to be connected to the one of the authorized roles for this
      # command, or the authorized role has to be casted to the default group of meta model where
      # the role interface is located.
      #
      # @param [String, UU::OS::UESURI] personal_access_role_uri UESURI of the personal access role where the role interface will be connected
      # @param [PersonalAccessRoleAddRoleInterface] personal_access_role DTO containing role interface that to be connected
      # @option personal_access_role [String, UU::OS::UESURI] :role_interface_uri UESURI of the role interface that to be connected to the specified personal access role. When nil is set, the command fails.
      #
      # @example
      #   # Add role interface for specified access role.
      #   UU::OS::PersonalAccessRole.add_role_interface('ues:TERRITORY:PERSONAL_ACCESS_ROLE', :role_interface_uri=>'ues:TERRITORY:ROLE_IFC')
      #
      def add_role_interface(personal_access_role_uri, personal_access_role = nil)
        svc = UU::OS::REST::RemoteClient.new(PersonalAccessRole)
        payload = UU::OS::PersonalAccessRole::PersonalAccessRoleAddRoleInterface.new(personal_access_role).to_json

        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('addRoleInterface', personal_access_role_uri, payload)
        end
      end

      # This command deactivates a personal access role. The personal access role will lose access to its
      # territory.
      #
      # @param [String, UU::OS::UESURI] personal_access_role_uri UESURI of a personal access role that the access will be canceled.
      #
      # @example
      #   # Deactivate access for specified access role.
      #   UU::OS::PersonalAccessRole.deactivate_access('ues:TERRITORY:PERSONAL_ACCESS_ROLE')
      def deactivate_access(personal_access_role_uri)
        svc = UU::OS::REST::RemoteClient.new(PersonalAccessRole)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('deactivateAccess', personal_access_role_uri)
        end
      end

      # This command activates a personal access role. The personal access role will have access to its territory.
      #
      # @param [String, UU::OS::UESURI] personal_access_role_uri UESURI of a personal access role that the access will be reactivated.
      #
      # @example
      #   # Activate access for specified access role.
      #   UU::OS::PersonalAccessRole.activate_access('ues:TERRITORY:PERSONAL_ACCESS_ROLE')
      #
      def activate_access(personal_access_role_uri)
        svc = UU::OS::REST::RemoteClient.new(PersonalAccessRole)
        UU::OS::QoS::QoSHandler.auto_retry do
          svc.post('activateAccess', personal_access_role_uri)
        end
      end

    end
  end

end
